<?php
/**
 * Created by PhpStorm.
 * User: robin
 * Date: 6/10/16
 * Time: 2:37 PM
 */

namespace Luticate\Utils\DataAccess;

use Illuminate\Database\Connection;
use Luticate\Utils\Business\LuStringUtils;
use Luticate\Utils\Dbo\LuDbo;
use Illuminate\Database\Capsule\Manager as Capsule;
use Luticate\Utils\Dbo\LuDboArray;
use Luticate\Utils\Dbo\LuPaginatedDbo;

abstract class LuStoredProcedure
{
    protected static $_connection = "mydb";
    protected static $_storedProcedure = "mydb_table";
    protected static $_dboClass = LuDbo::class;
    protected static $_dboArrayClass = null;

    /**
     * @return Connection
     */
    public static function getConnection()
    {
        return Capsule::connection(static::$_connection);
    }
    
    public static function getDboClassArray()
    {
        if (is_null(static::$_dboArrayClass)) {
            return static::$_dboClass . "[]";
        }
        return static::$_dboArrayClass;
    }

    public static function getArgsArray($args)
    {
        $argsArray = [];
        for ($i = 0; $i < count($args); ++$i) {
            $argsArray[":_arg" . $i] = $args[$i];
        }
        return $argsArray;
    }

    public static function getSqlSelectQuery($args)
    {
        $storedProcedure = static::$_storedProcedure;
        $argsString = implode(", ", array_map(function($i)
        {
            return ":_arg" . $i;
        }, range(0, count($args) - 1)));
        $query = "SELECT COALESCE(json_agg(data), '[]'::json) as data FROM ${storedProcedure}(${argsString}) data";
        return $query;
    }

    /**
     * @param $args array
     * @return LuDbo[]
     */
    public static function getMultiple($args)
    {
        $argsArray = static::getArgsArray($args);
        $query = static::getSqlSelectQuery($args);
        $values = static::getConnection()->select($query, $argsArray);

        $value = $values[0]->data;
        $json = LuStringUtils::convertJsonString($value);
        $dboValues = LuDbo::deserializeValue($json, static::getDboClassArray());
        if ($dboValues instanceof LuDboArray) {
            return $dboValues->getArray();
        }
        return $dboValues;
    }

    /**
     * @param $args array
     * @return LuDbo|null
     */
    public static function getSingle($args)
    {
        $argsArray = static::getArgsArray($args);
        $query = static::getSqlSelectQuery($args);
        $values = static::getConnection()->select($query, $argsArray);

        $value = $values[0]->data;
        $json = LuStringUtils::convertJsonString($value);
        if (count($json) < 1 || is_null($json[0])) {
            return null;
        }
        $dboValue = LuDbo::deserializeValue($json, static::getDboClassArray());
        return $dboValue[0];
    }

    /**
     * @param $args array
     * @param $page int The page number, 0 based
     * @param $perPage int The number of items per page
     * @return LuPaginatedDbo
     */
    public static function getMultiplePaginated($args, $page, $perPage)
    {
        $argsArray = static::getArgsArray($args);
        $argsArray[":_page"] = $page;
        $argsArray[":_perPage"] = $perPage;

        $storedProcedure = static::$_storedProcedure;
        $argsString = implode(", ", array_map(function($i)
        {
            return ":_arg" . $i;
        }, range(0, count($args) - 1)));

        $query = "SELECT (SELECT count(*) FROM ${storedProcedure}(${argsString})) as count,
          (SELECT COALESCE(json_agg(data), '[]'::json) FROM (SELECT data FROM ${storedProcedure}(${argsString}) data OFFSET (:_page::int * :_perPage::int) LIMIT :_perPage) data) as data";
        $values = static::getConnection()->select($query, $argsArray);

        $value = $values[0];
        if (is_null($value->data))
        {
            $value->data = '[]';
        }
        $json = LuStringUtils::convertJsonString($value->data);
        $dboValues = LuDbo::deserializeValue($json, static::getDboClassArray());
        if ($dboValues instanceof LuDboArray) {
            $dboValues = $dboValues->getArray();
        }
        return new LuPaginatedDbo($value->count, $dboValues);
    }
}